@@ -22,16 +22,26 @@ public:
         return fGM->modifyGrContextOptions(options);
     }
 
+#if defined(SK_GRAPHITE)
+    void modifyGraphiteContextOptions(skgpu::graphite::ContextOptions* options) override {
+        fGM->modifyGraphiteContextOptions(options);
+    }
+#endif
+
 protected:
     const char* onGetName() override;
     bool isSuitableFor(Backend backend) override;
-    void onDraw(int loops, SkCanvas* canvas) override;
-    SkIPoint onGetSize() override;
+    void onPerCanvasPreDraw(SkCanvas*) override;
+    void onPerCanvasPostDraw(SkCanvas*) override;
+    void onDraw(int loops, SkCanvas*) override;
+    SkISize onGetSize() override;
 
 private:
     std::unique_ptr<skiagm::GM> fGM;
-    SkString    fName;
-    typedef Benchmark INHERITED;
+    SkString                    fName;
+    bool                        fGpuSetupFailed = false;
+
+    using INHERITED = Benchmark;
 };
 
 #endif
