@@ -9,6 +9,7 @@
 #include <stdint.h>
 
 #include <algorithm>
+#include <array>
 #include <limits>
 #include <memory>
 #include <utility>
@@ -38,6 +39,7 @@
 #include "core/fxcrt/ptr_util.h"
 #include "core/fxcrt/span.h"
 #include "core/fxcrt/stl_util.h"
+#include "core/fxge/agg/cfx_agg_imagerenderer.h"
 #include "core/fxge/cfx_defaultrenderdevice.h"
 #include "core/fxge/cfx_fillrenderoptions.h"
 #include "core/fxge/cfx_font.h"
@@ -47,7 +49,6 @@
 #include "core/fxge/cfx_substfont.h"
 #include "core/fxge/cfx_textrenderoptions.h"
 #include "core/fxge/dib/cfx_dibitmap.h"
-#include "core/fxge/dib/cfx_imagerenderer.h"
 #include "core/fxge/dib/cstretchengine.h"
 #include "core/fxge/dib/fx_dib.h"
 #include "core/fxge/text_char_pos.h"
@@ -125,10 +126,9 @@ void DebugShowCanvasClip(CFX_SkiaDeviceDriver* driver, const SkCanvas* canvas) {
          local.fBottom);
   printf("device bounds %d %d %d %d\n", device.fLeft, device.fTop,
          device.fRight, device.fBottom);
-  FX_RECT clipBox;
-  driver->GetClipBox(&clipBox);
-  printf("reported bounds %d %d %d %d\n", clipBox.left, clipBox.top,
-         clipBox.right, clipBox.bottom);
+  FX_RECT clip_box = driver->GetClipBox();
+  printf("reported bounds %d %d %d %d\n", clip_box.left, clip_box.top,
+         clip_box.right, clip_box.bottom);
 #endif  // SHOW_SKIA_PATH
 }
 
@@ -342,27 +342,28 @@ bool AddSamples(const CPDF_SampledFunc* func,
     return false;
   }
 
-  float colors_min[3];
-  float colors_max[3];
+  std::array<float, 3> colors_min;
+  std::array<float, 3> colors_max;
   for (int i = 0; i < 3; ++i) {
     colors_min[i] = func->GetRange(i * 2);
     colors_max[i] = func->GetRange(i * 2 + 1);
   }
-  pdfium::span<const uint8_t> sample_data = func->GetSampleStream()->GetSpan();
-  CFX_BitStream bitstream(sample_data);
-  for (uint32_t i = 0; i < sample_count; ++i) {
-    float float_colors[3];
-    for (uint32_t j = 0; j < 3; ++j) {
-      float sample = static_cast<float>(bitstream.GetBits(sample_size));
-      float interp = sample / (sample_count - 1);
-      float_colors[j] =
-          colors_min[j] + (colors_max[j] - colors_min[j]) * interp;
+    pdfium::span<const uint8_t> sample_data =
+        func->GetSampleStream()->GetSpan();
+    CFX_BitStream bitstream(sample_data);
+    for (uint32_t i = 0; i < sample_count; ++i) {
+      std::array<float, 3> float_colors;
+      for (uint32_t j = 0; j < 3; ++j) {
+        float sample = static_cast<float>(bitstream.GetBits(sample_size));
+        float interp = sample / (sample_count - 1);
+        float_colors[j] =
+            colors_min[j] + (colors_max[j] - colors_min[j]) * interp;
+      }
+      colors.push_back(SkPackARGB32(0xFF, FloatToByte(float_colors[0]),
+                                    FloatToByte(float_colors[1]),
+                                    FloatToByte(float_colors[2])));
+      pos.push_back(static_cast<float>(i) / (sample_count - 1));
     }
-    colors.push_back(SkPackARGB32NoCheck(0xFF, FloatToByte(float_colors[0]),
-                                         FloatToByte(float_colors[1]),
-                                         FloatToByte(float_colors[2])));
-    pos.push_back(static_cast<float>(i) / (sample_count - 1));
-  }
   return true;
 }
 
@@ -393,9 +394,12 @@ bool AddStitching(const CPDF_StitchFunc* func,
 }
 
 // see https://en.wikipedia.org/wiki/Distance_from_a_point_to_a_line
-SkScalar LineSide(const SkPoint line[2], const SkPoint& pt) {
-  return (line[1].fY - line[0].fY) * pt.fX - (line[1].fX - line[0].fX) * pt.fY +
-         line[1].fX * line[0].fY - line[1].fY * line[0].fX;
+SkScalar LineSide(const SkPoint& line_start,
+                  const SkPoint& line_end,
+                  const SkPoint& pt) {
+  return (line_end.fY - line_start.fY) * pt.fX -
+         (line_end.fX - line_start.fX) * pt.fY + line_end.fX * line_start.fY -
+         line_end.fY * line_start.fX;
 }
 
 SkPoint IntersectSides(const SkPoint& parallelPt,
@@ -415,10 +419,10 @@ SkPoint IntersectSides(const SkPoint& parallelPt,
   return result;
 }
 
-void ClipAngledGradient(const SkPoint pts[2],
-                        SkPoint rectPts[4],
-                        bool clipStart,
-                        bool clipEnd,
+void ClipAngledGradient(pdfium::span<const SkPoint, 2> pts,
+                        pdfium::span<const SkPoint, 4> rect_pts,
+                        bool clip_start,
+                        bool clip_end,
                         SkPath* clip) {
   // find the corners furthest from the gradient perpendiculars
   SkScalar minPerpDist = SK_ScalarMax;
@@ -426,13 +430,16 @@ void ClipAngledGradient(const SkPoint pts[2],
   int minPerpPtIndex = -1;
   int maxPerpPtIndex = -1;
   SkVector slope = pts[1] - pts[0];
-  SkPoint startPerp[2] = {pts[0], {pts[0].fX + slope.fY, pts[0].fY - slope.fX}};
-  SkPoint endPerp[2] = {pts[1], {pts[1].fX + slope.fY, pts[1].fY - slope.fX}};
+  const SkPoint start_perp[2] = {pts[0],
+                                 {pts[0].fX + slope.fY, pts[0].fY - slope.fX}};
+  const SkPoint end_perp[2] = {pts[1],
+                               {pts[1].fX + slope.fY, pts[1].fY - slope.fX}};
   for (int i = 0; i < 4; ++i) {
-    SkScalar sDist = LineSide(startPerp, rectPts[i]);
-    SkScalar eDist = LineSide(endPerp, rectPts[i]);
-    if (sDist * eDist <= 0)  // if the signs are different,
-      continue;              // the point is inside the gradient
+    SkScalar sDist = LineSide(start_perp[0], start_perp[1], rect_pts[i]);
+    SkScalar eDist = LineSide(end_perp[0], end_perp[1], rect_pts[i]);
+    if (sDist * eDist <= 0) {  // if the signs are different,
+      continue;                // the point is inside the gradient
+    }
     if (sDist < 0) {
       SkScalar smaller = std::min(sDist, eDist);
       if (minPerpDist > smaller) {
@@ -447,24 +454,27 @@ void ClipAngledGradient(const SkPoint pts[2],
       }
     }
   }
-  if (minPerpPtIndex < 0 && maxPerpPtIndex < 0)  // nothing's outside
-    return;
-
-  // determine if negative distances are before start or after end
-  SkPoint beforeStart = {pts[0].fX * 2 - pts[1].fX, pts[0].fY * 2 - pts[1].fY};
-  bool beforeNeg = LineSide(startPerp, beforeStart) < 0;
-
-  int noClipStartIndex = maxPerpPtIndex;
-  int noClipEndIndex = minPerpPtIndex;
-  if (beforeNeg)
-    std::swap(noClipStartIndex, noClipEndIndex);
-  if ((!clipStart && noClipStartIndex < 0) ||
-      (!clipEnd && noClipEndIndex < 0)) {
+  if (minPerpPtIndex < 0 && maxPerpPtIndex < 0) {  // nothing's outside
     return;
   }
 
-  const SkPoint& startEdgePt = clipStart ? pts[0] : rectPts[noClipStartIndex];
-  const SkPoint& endEdgePt = clipEnd ? pts[1] : rectPts[noClipEndIndex];
+  // determine if negative distances are before start or after end
+  const SkPoint before_start = {pts[0].fX * 2 - pts[1].fX,
+                                pts[0].fY * 2 - pts[1].fY};
+  bool before_neg = LineSide(start_perp[0], start_perp[1], before_start) < 0;
+
+  int noClipStartIndex = maxPerpPtIndex;
+  int noClipEndIndex = minPerpPtIndex;
+  if (before_neg) {
+    std::swap(noClipStartIndex, noClipEndIndex);
+  }
+  if ((!clip_start && noClipStartIndex < 0) ||
+      (!clip_end && noClipEndIndex < 0)) {
+    return;
+  }
+
+  const SkPoint& startEdgePt = clip_start ? pts[0] : rect_pts[noClipStartIndex];
+  const SkPoint& endEdgePt = clip_end ? pts[1] : rect_pts[noClipEndIndex];
 
   // find the corners that bound the gradient
   SkScalar minDist = SK_ScalarMax;
@@ -472,7 +482,7 @@ void ClipAngledGradient(const SkPoint pts[2],
   int minBounds = -1;
   int maxBounds = -1;
   for (int i = 0; i < 4; ++i) {
-    SkScalar dist = LineSide(pts, rectPts[i]);
+    SkScalar dist = LineSide(pts[0], pts[1], rect_pts[i]);
     if (minDist > dist) {
       minDist = dist;
       minBounds = i;
@@ -482,17 +492,19 @@ void ClipAngledGradient(const SkPoint pts[2],
       maxBounds = i;
     }
   }
-  if (minBounds < 0 || maxBounds < 0)
+  if (minBounds < 0 || maxBounds < 0) {
     return;
-  if (minBounds == maxBounds)
+  }
+  if (minBounds == maxBounds) {
     return;
+  }
   // construct a clip parallel to the gradient that goes through
-  // rectPts[minBounds] and rectPts[maxBounds] and perpendicular to the
+  // rect_pts[minBounds] and rect_pts[maxBounds] and perpendicular to the
   // gradient that goes through startEdgePt, endEdgePt.
-  clip->moveTo(IntersectSides(rectPts[minBounds], slope, startEdgePt));
-  clip->lineTo(IntersectSides(rectPts[minBounds], slope, endEdgePt));
-  clip->lineTo(IntersectSides(rectPts[maxBounds], slope, endEdgePt));
-  clip->lineTo(IntersectSides(rectPts[maxBounds], slope, startEdgePt));
+  clip->moveTo(IntersectSides(rect_pts[minBounds], slope, startEdgePt));
+  clip->lineTo(IntersectSides(rect_pts[minBounds], slope, endEdgePt));
+  clip->lineTo(IntersectSides(rect_pts[maxBounds], slope, endEdgePt));
+  clip->lineTo(IntersectSides(rect_pts[maxBounds], slope, startEdgePt));
 }
 
 // Converts a stroking path to scanlines
@@ -501,7 +513,7 @@ void PaintStroke(SkPaint* spaint,
                  const SkMatrix& matrix,
                  const CFX_FillRenderOptions& fill_options) {
   SkPaint::Cap cap;
-  switch (graph_state->m_LineCap) {
+  switch (graph_state->line_cap()) {
     case CFX_GraphStateData::LineCap::kRound:
       cap = SkPaint::kRound_Cap;
       break;
@@ -513,7 +525,7 @@ void PaintStroke(SkPaint* spaint,
       break;
   }
   SkPaint::Join join;
-  switch (graph_state->m_LineJoin) {
+  switch (graph_state->line_join()) {
     case CFX_GraphStateData::LineJoin::kRound:
       join = SkPaint::kRound_Join;
       break;
@@ -535,33 +547,32 @@ void PaintStroke(SkPaint* spaint,
 
   float width = fill_options.zero_area
                     ? 0.0f
-                    : std::max(graph_state->m_LineWidth,
+                    : std::max(graph_state->line_width(),
                                std::min(deviceUnits[0].length(),
                                         deviceUnits[1].length()));
-  if (!graph_state->m_DashArray.empty()) {
-    size_t count = (graph_state->m_DashArray.size() + 1) / 2;
+  const std::vector<float>& dash_array = graph_state->dash_array();
+  if (!dash_array.empty()) {
+    size_t count = (dash_array.size() + 1) / 2;
     DataVector<SkScalar> intervals(count * 2);
     // Set dash pattern
     for (size_t i = 0; i < count; i++) {
-      float on = graph_state->m_DashArray[i * 2];
+      float on = dash_array[i * 2];
       if (on <= 0.000001f) {
         on = 0.1f;
       }
-      float off = i * 2 + 1 == graph_state->m_DashArray.size()
-                      ? on
-                      : graph_state->m_DashArray[i * 2 + 1];
+      float off = i * 2 + 1 == dash_array.size() ? on : dash_array[i * 2 + 1];
       off = std::max(off, 0.0f);
       intervals[i * 2] = on;
       intervals[i * 2 + 1] = off;
     }
     spaint->setPathEffect(SkDashPathEffect::Make(
         intervals.data(), pdfium::checked_cast<int>(intervals.size()),
-        graph_state->m_DashPhase));
+        graph_state->dash_phase()));
   }
   spaint->setStyle(SkPaint::kStroke_Style);
   spaint->setAntiAlias(!fill_options.aliased_path);
   spaint->setStrokeWidth(width);
-  spaint->setStrokeMiter(graph_state->m_MiterLimit);
+  spaint->setStrokeMiter(graph_state->miter_limit());
   spaint->setStrokeCap(cap);
   spaint->setStrokeJoin(join);
 }
@@ -610,8 +621,11 @@ void SetBitmapPaintForMerge(bool is_mask,
 // Makes a bitmap filled with a solid color for debugging with `SkPicture`.
 RetainPtr<CFX_DIBitmap> MakeDebugBitmap(int width, int height, uint32_t color) {
   auto bitmap = pdfium::MakeRetain<CFX_DIBitmap>();
-  if (!bitmap->Create(width, height, FXDIB_Format::kArgb))
+  // TODO(crbug.com/42271020): Consider adding support for
+  // `FXDIB_Format::kBgraPremul`
+  if (!bitmap->Create(width, height, FXDIB_Format::kBgra)) {
     return nullptr;
+  }
 
   bitmap->Clear(color);
   return bitmap;
@@ -669,11 +683,7 @@ std::unique_ptr<CFX_SkiaDeviceDriver> CFX_SkiaDeviceDriver::Create(
 
 // static
 std::unique_ptr<CFX_SkiaDeviceDriver> CFX_SkiaDeviceDriver::Create(
-    SkCanvas* canvas) {
-  if (!canvas) {
-    return nullptr;
-  }
-
+    SkCanvas& canvas) {
   auto driver = pdfium::WrapUnique(new CFX_SkiaDeviceDriver(canvas));
   if (!driver->m_pBitmap || !driver->m_pBackdropBitmap) {
     return nullptr;
@@ -693,19 +703,24 @@ CFX_SkiaDeviceDriver::CFX_SkiaDeviceDriver(
       m_bGroupKnockout(bGroupKnockout) {
   SkColorType color_type;
   const int bpp = m_pBitmap->GetBPP();
+  SkAlphaType alpha_type = kPremul_SkAlphaType;
   if (bpp == 8) {
     color_type = m_pBitmap->IsAlphaFormat() || m_pBitmap->IsMaskFormat()
                      ? kAlpha_8_SkColorType
                      : kGray_8_SkColorType;
   } else if (bpp == 24) {
-    DCHECK_EQ(m_pBitmap->GetFormat(), FXDIB_Format::kRgb);
+    DCHECK_EQ(m_pBitmap->GetFormat(), FXDIB_Format::kBgr);
 
     // Save the input bitmap as `m_pOriginalBitmap` and save its 32 bpp
     // equivalent at `m_pBitmap` for Skia's internal process.
     m_pOriginalBitmap = std::move(m_pBitmap);
+    const int width = m_pOriginalBitmap->GetWidth();
+    const int height = m_pOriginalBitmap->GetHeight();
+
     m_pBitmap = pdfium::MakeRetain<CFX_DIBitmap>();
-    if (!m_pBitmap->Copy(m_pOriginalBitmap) ||
-        !m_pBitmap->ConvertFormat(FXDIB_Format::kArgb)) {
+    if (!m_pBitmap->Create(width, height, FXDIB_Format::kBgraPremul) ||
+        !m_pBitmap->TransferBitmap(width, height, m_pOriginalBitmap,
+                                   /*src_left=*/0, /*src_top=*/0)) {
       // Skip creating SkCanvas if the 32-bpp bitmap creation fails.
       // CFX_SkiaDeviceDriver::Create() will check for the missing `m_pCanvas`
       // and not use `this`.
@@ -719,34 +734,37 @@ CFX_SkiaDeviceDriver::CFX_SkiaDeviceDriver(
   } else {
     DCHECK_EQ(bpp, 32);
     color_type = Get32BitSkColorType(bRgbByteOrder);
+    FXDIB_Format format = m_pBitmap->GetFormat();
+    if (format == FXDIB_Format::kBgrx) {
+      alpha_type = kOpaque_SkAlphaType;
+    } else if (format == FXDIB_Format::kBgra) {
+      alpha_type = kUnpremul_SkAlphaType;
+    }
   }
 
-  SkImageInfo imageInfo =
-      SkImageInfo::Make(m_pBitmap->GetWidth(), m_pBitmap->GetHeight(),
-                        color_type, kPremul_SkAlphaType);
+  SkImageInfo imageInfo = SkImageInfo::Make(
+      m_pBitmap->GetWidth(), m_pBitmap->GetHeight(), color_type, alpha_type);
   surface_ = SkSurfaces::WrapPixels(
       imageInfo, m_pBitmap->GetWritableBuffer().data(), m_pBitmap->GetPitch());
   m_pCanvas = surface_->getCanvas();
 }
 
-CFX_SkiaDeviceDriver::CFX_SkiaDeviceDriver(SkCanvas* canvas)
-    : m_pCanvas(canvas), m_bGroupKnockout(false) {
+CFX_SkiaDeviceDriver::CFX_SkiaDeviceDriver(SkCanvas& canvas)
+    : m_pCanvas(&canvas), m_bRgbByteOrder(false), m_bGroupKnockout(false) {
   int width = m_pCanvas->imageInfo().width();
   int height = m_pCanvas->imageInfo().height();
   DCHECK_EQ(kUnknown_SkColorType, m_pCanvas->imageInfo().colorType());
 
-  constexpr uint32_t kMagenta = 0xffff00ff;
-  constexpr uint32_t kGreen = 0xff00ff00;
+  static constexpr uint32_t kMagenta = 0xffff00ff;
+  static constexpr uint32_t kGreen = 0xff00ff00;
   m_pBitmap = MakeDebugBitmap(width, height, kMagenta);
   m_pBackdropBitmap = MakeDebugBitmap(width, height, kGreen);
 }
 
 CFX_SkiaDeviceDriver::~CFX_SkiaDeviceDriver() {
   // Convert and transfer the internal processed result to the original 24 bpp
-  // bitmap provided by the render device.
-  if (m_pOriginalBitmap && m_pBitmap->ConvertFormat(FXDIB_Format::kRgb)) {
-    CHECK(SyncInternalBitmaps());
-  }
+  // bitmap provided by the render device if needed.
+  SyncInternalBitmaps();
 }
 
 bool CFX_SkiaDeviceDriver::DrawDeviceText(
@@ -854,30 +872,24 @@ bool CFX_SkiaDeviceDriver::TryDrawText(pdfium::span<const TextCharPos> char_pos,
     return false;
   }
 
-  m_charDetails.SetCount(0);
-  m_rsxform.resize(0);
-
-  const size_t original_count = m_charDetails.Count();
-  FX_SAFE_SIZE_T safe_count = original_count;
-  safe_count += char_pos.size();
-  const size_t total_count = safe_count.ValueOrDie();
-  m_charDetails.SetCount(total_count);
+  m_charDetails.SetCount(char_pos.size());
   if (hasRSX) {
-    m_rsxform.resize(total_count);
+    m_rsxform.resize(char_pos.size());
+  } else {
+    m_rsxform.resize(0);
   }
 
   const SkScalar horizontal_flip = font_size < 0 ? -1 : 1;
   const SkScalar vertical_flip = pFont->IsVertical() ? -1 : 1;
   for (size_t index = 0; index < char_pos.size(); ++index) {
     const TextCharPos& cp = char_pos[index];
-    size_t cur_index = index + original_count;
-    m_charDetails.SetPositionAt(cur_index, {cp.m_Origin.x * horizontal_flip,
-                                            cp.m_Origin.y * vertical_flip});
-    m_charDetails.SetGlyphAt(cur_index, static_cast<uint16_t>(cp.m_GlyphIndex));
-    m_charDetails.SetFontCharWidthAt(cur_index, cp.m_FontCharWidth);
+    m_charDetails.SetPositionAt(index, {cp.m_Origin.x * horizontal_flip,
+                                        cp.m_Origin.y * vertical_flip});
+    m_charDetails.SetGlyphAt(index, static_cast<uint16_t>(cp.m_GlyphIndex));
+    m_charDetails.SetFontCharWidthAt(index, cp.m_FontCharWidth);
 #if BUILDFLAG(IS_APPLE)
     if (cp.m_ExtGID) {
-      m_charDetails.SetGlyphAt(cur_index, static_cast<uint16_t>(cp.m_ExtGID));
+      m_charDetails.SetGlyphAt(index, static_cast<uint16_t>(cp.m_ExtGID));
     }
 #endif
   }
@@ -885,7 +897,7 @@ bool CFX_SkiaDeviceDriver::TryDrawText(pdfium::span<const TextCharPos> char_pos,
     const DataVector<SkPoint>& positions = m_charDetails.GetPositions();
     for (size_t index = 0; index < char_pos.size(); ++index) {
       const TextCharPos& cp = char_pos[index];
-      SkRSXform& rsxform = m_rsxform[index + original_count];
+      SkRSXform& rsxform = m_rsxform[index];
       if (cp.m_bGlyphAdjust) {
         rsxform.fSCos = cp.m_AdjustMatrix[0];
         rsxform.fSSin = cp.m_AdjustMatrix[1];
@@ -1001,9 +1013,9 @@ int CFX_SkiaDeviceDriver::GetDeviceCaps(int caps_id) const {
     case FXDC_RENDER_CAPS:
       return FXRC_GET_BITS | FXRC_ALPHA_PATH | FXRC_ALPHA_IMAGE |
              FXRC_BLEND_MODE | FXRC_SOFT_CLIP | FXRC_ALPHA_OUTPUT |
-             FXRC_FILLSTROKE_PATH | FXRC_SHADING;
+             FXRC_FILLSTROKE_PATH | FXRC_SHADING | FXRC_PREMULTIPLIED_ALPHA;
     default:
-      NOTREACHED_NORETURN();
+      NOTREACHED();
   }
 }
 
@@ -1068,15 +1080,12 @@ bool CFX_SkiaDeviceDriver::SetClip_PathStroke(
   return true;
 }
 
-// TODO(crbug.com/pdfium/1963): `blend_type` isn't used?
-bool CFX_SkiaDeviceDriver::DrawPath(
-    const CFX_Path& path,                   // path info
-    const CFX_Matrix* pObject2Device,       // optional transformation
-    const CFX_GraphStateData* pGraphState,  // graphic state, for pen attributes
-    uint32_t fill_color,                    // fill color
-    uint32_t stroke_color,                  // stroke color
-    const CFX_FillRenderOptions& fill_options,
-    BlendMode blend_type) {
+bool CFX_SkiaDeviceDriver::DrawPath(const CFX_Path& path,
+                                    const CFX_Matrix* pObject2Device,
+                                    const CFX_GraphStateData* pGraphState,
+                                    uint32_t fill_color,
+                                    uint32_t stroke_color,
+                                    const CFX_FillRenderOptions& fill_options) {
   m_FillOptions = fill_options;
 
   SkPath skia_path = BuildPath(path);
@@ -1117,8 +1126,7 @@ bool CFX_SkiaDeviceDriver::DrawPath(
     }
     skPaint.setStyle(SkPaint::kFill_Style);
     skPaint.setColor(fill_color);
-    DebugShowSkiaDrawPath(this, m_pCanvas, skPaint, *fillPath);
-    m_pCanvas->drawPath(*fillPath, skPaint);
+    DrawPathImpl(*fillPath, skPaint);
   }
   if (stroke_alpha && do_stroke) {
     skPaint.setStyle(SkPaint::kStroke_Style);
@@ -1131,27 +1139,16 @@ bool CFX_SkiaDeviceDriver::DrawPath(
       // Do nothing. A closed 0-length closed path can be rendered only if
       // its line cap type is round.
     } else {
-      DebugShowSkiaDrawPath(this, m_pCanvas, skPaint, skia_path);
-      m_pCanvas->drawPath(skia_path, skPaint);
+      DrawPathImpl(skia_path, skPaint);
     }
   }
   return true;
 }
 
-bool CFX_SkiaDeviceDriver::DrawCosmeticLine(const CFX_PointF& ptMoveTo,
-                                            const CFX_PointF& ptLineTo,
-                                            uint32_t color,
-                                            BlendMode blend_type) {
-  return false;
-}
-
-bool CFX_SkiaDeviceDriver::FillRectWithBlend(const FX_RECT& rect,
-                                             uint32_t fill_color,
-                                             BlendMode blend_type) {
+bool CFX_SkiaDeviceDriver::FillRect(const FX_RECT& rect, uint32_t fill_color) {
   SkPaint spaint;
   spaint.setAntiAlias(true);
   spaint.setColor(fill_color);
-  spaint.setBlendMode(GetSkiaBlendMode(blend_type));
   SkRect srect = SkRect::MakeLTRB(rect.left, std::min(rect.top, rect.bottom),
                                   rect.right, std::max(rect.bottom, rect.top));
   DebugShowSkiaDrawRect(this, m_pCanvas, spaint, srect);
@@ -1159,32 +1156,32 @@ bool CFX_SkiaDeviceDriver::FillRectWithBlend(const FX_RECT& rect,
   return true;
 }
 
-bool CFX_SkiaDeviceDriver::DrawShading(const CPDF_ShadingPattern* pPattern,
-                                       const CFX_Matrix* pMatrix,
+bool CFX_SkiaDeviceDriver::DrawShading(const CPDF_ShadingPattern& pattern,
+                                       const CFX_Matrix& matrix,
                                        const FX_RECT& clip_rect,
-                                       int alpha,
-                                       bool bAlphaMode) {
-  ShadingType shadingType = pPattern->GetShadingType();
-  if (kAxialShading != shadingType && kRadialShading != shadingType &&
-      kCoonsPatchMeshShading != shadingType) {
+                                       int alpha) {
+  const ShadingType shading_type = pattern.GetShadingType();
+  if (shading_type != kAxialShading && shading_type != kRadialShading &&
+      shading_type != kCoonsPatchMeshShading) {
     // TODO(caryclark) more types
     return false;
   }
-  CPDF_ColorSpace::Family csFamily = pPattern->GetCS()->GetFamily();
-  if (CPDF_ColorSpace::Family::kDeviceRGB != csFamily &&
-      CPDF_ColorSpace::Family::kDeviceGray != csFamily) {
+  CPDF_ColorSpace::Family cs_family = pattern.GetCS()->GetFamily();
+  if (CPDF_ColorSpace::Family::kDeviceRGB != cs_family &&
+      CPDF_ColorSpace::Family::kDeviceGray != cs_family) {
     return false;
   }
   const std::vector<std::unique_ptr<CPDF_Function>>& pFuncs =
-      pPattern->GetFuncs();
+      pattern.GetFuncs();
   size_t nFuncs = pFuncs.size();
   if (nFuncs > 1)  // TODO(caryclark) remove this restriction
     return false;
   RetainPtr<const CPDF_Dictionary> pDict =
-      pPattern->GetShadingObject()->GetDict();
+      pattern.GetShadingObject()->GetDict();
   RetainPtr<const CPDF_Array> pCoords = pDict->GetArrayFor("Coords");
-  if (!pCoords && kCoonsPatchMeshShading != shadingType)
+  if (!pCoords && shading_type != kCoonsPatchMeshShading) {
     return false;
+  }
   // TODO(caryclark) Respect Domain[0], Domain[1]. (Don't know what they do
   // yet.)
   DataVector<SkColor> sk_colors;
@@ -1221,12 +1218,12 @@ bool CFX_SkiaDeviceDriver::DrawShading(const CPDF_ShadingPattern* pPattern,
   SkPaint paint;
   paint.setAntiAlias(true);
   paint.setAlpha(alpha);
-  SkMatrix skMatrix = ToSkMatrix(*pMatrix);
+  SkMatrix skMatrix = ToSkMatrix(matrix);
   SkRect skRect = SkRect::MakeLTRB(clip_rect.left, clip_rect.top,
                                    clip_rect.right, clip_rect.bottom);
   SkPath skClip;
   SkPath skPath;
-  if (kAxialShading == shadingType) {
+  if (shading_type == kAxialShading) {
     float start_x = pCoords->GetFloatAt(0);
     float start_y = pCoords->GetFloatAt(1);
     float end_x = pCoords->GetFloatAt(2);
@@ -1266,7 +1263,7 @@ bool CFX_SkiaDeviceDriver::DrawShading(const CPDF_ShadingPattern* pPattern,
     }
     skPath.addRect(skRect);
     skMatrix.setIdentity();
-  } else if (kRadialShading == shadingType) {
+  } else if (shading_type == kRadialShading) {
     float start_x = pCoords->GetFloatAt(0);
     float start_y = pCoords->GetFloatAt(1);
     float start_r = pCoords->GetFloatAt(2);
@@ -1293,48 +1290,46 @@ bool CFX_SkiaDeviceDriver::DrawShading(const CPDF_ShadingPattern* pPattern,
     skPath.addRect(skRect);
     skPath.transform(inverse);
   } else {
-    DCHECK_EQ(kCoonsPatchMeshShading, shadingType);
-    RetainPtr<const CPDF_Stream> pStream =
-        ToStream(pPattern->GetShadingObject());
+    CHECK_EQ(kCoonsPatchMeshShading, shading_type);
+    RetainPtr<const CPDF_Stream> pStream = ToStream(pattern.GetShadingObject());
     if (!pStream)
       return false;
-    CPDF_MeshStream stream(shadingType, pPattern->GetFuncs(),
-                           std::move(pStream), pPattern->GetCS());
+    CPDF_MeshStream stream(shading_type, pattern.GetFuncs(), std::move(pStream),
+                           pattern.GetCS());
     if (!stream.Load())
       return false;
-    SkPoint cubics[12];
-    SkColor colors[4];
+    std::array<SkPoint, 12> cubics;
+    std::array<SkColor, 4> colors;
     SkAutoCanvasRestore scoped_save_restore(m_pCanvas, /*doSave=*/true);
     if (!skClip.isEmpty())
       m_pCanvas->clipPath(skClip, SkClipOp::kIntersect, true);
     m_pCanvas->concat(skMatrix);
     while (!stream.IsEOF()) {
-      uint32_t flag = stream.ReadFlag();
-      size_t start_point = flag ? 4 : 0;
-      size_t start_color = flag ? 2 : 0;
+      const uint32_t flag = stream.ReadFlag();
       if (flag) {
-        SkPoint temp_cubics[4];
-        for (size_t i = 0; i < std::size(temp_cubics); ++i) {
-          temp_cubics[i] = cubics[(flag * 3 + i) % 12];
+        std::array<SkPoint, 4> temp_cubics;
+        for (size_t i = 0; i < temp_cubics.size(); ++i) {
+          temp_cubics[i] = cubics[(flag * 3 + i) % cubics.size()];
         }
-        std::copy(std::begin(temp_cubics), std::end(temp_cubics),
-                  std::begin(cubics));
-        SkColor temp_colors[2] = {colors[flag % 4], colors[(flag + 1) % 4]};
-        std::copy(std::begin(temp_colors), std::end(temp_colors),
-                  std::begin(colors));
+        fxcrt::Copy(temp_cubics, cubics);
+        SkColor temp_colors[2] = {colors[flag % 4],
+                                  colors[(flag + 1) % colors.size()]};
+        fxcrt::Copy(temp_colors, colors);
       }
-      for (size_t i = start_point; i < std::size(cubics); ++i) {
+      const size_t start_point = flag ? 4 : 0;
+      for (size_t i = start_point; i < cubics.size(); ++i) {
         CFX_PointF point = stream.ReadCoords();
         cubics[i].fX = point.x;
         cubics[i].fY = point.y;
       }
-      for (size_t i = start_color; i < std::size(colors); ++i) {
-        FX_RGB<float> rgb = stream.ReadColor();
+      const size_t start_color = flag ? 2 : 0;
+      for (size_t i = start_color; i < colors.size(); ++i) {
+        FX_RGB_STRUCT<float> rgb = stream.ReadColor();
         colors[i] =
             SkColorSetARGB(0xFF, (U8CPU)(rgb.red * 255),
                            (U8CPU)(rgb.green * 255), (U8CPU)(rgb.blue * 255));
       }
-      m_pCanvas->drawPatch(cubics, colors, /*texCoords=*/nullptr,
+      m_pCanvas->drawPatch(cubics.data(), colors.data(), /*texCoords=*/nullptr,
                            SkBlendMode::kDst, paint);
     }
     return true;
@@ -1343,22 +1338,18 @@ bool CFX_SkiaDeviceDriver::DrawShading(const CPDF_ShadingPattern* pPattern,
   if (!skClip.isEmpty())
     m_pCanvas->clipPath(skClip, SkClipOp::kIntersect, true);
   m_pCanvas->concat(skMatrix);
-  m_pCanvas->drawPath(skPath, paint);
+  DrawPathImpl(skPath, paint);
   return true;
 }
 
-bool CFX_SkiaDeviceDriver::GetClipBox(FX_RECT* pRect) {
+FX_RECT CFX_SkiaDeviceDriver::GetClipBox() const {
   SkIRect clip = m_pCanvas->getDeviceClipBounds();
-  pRect->left = clip.fLeft;
-  pRect->top = clip.fTop;
-  pRect->right = clip.fRight;
-  pRect->bottom = clip.fBottom;
-  return true;
+  return FX_RECT(clip.fLeft, clip.fTop, clip.fRight, clip.fBottom);
 }
 
 bool CFX_SkiaDeviceDriver::GetDIBits(RetainPtr<CFX_DIBitmap> bitmap,
                                      int left,
-                                     int top) {
+                                     int top) const {
   const uint8_t* input_buffer = m_pBitmap->GetBuffer().data();
   if (!input_buffer) {
     return true;
@@ -1367,16 +1358,16 @@ bool CFX_SkiaDeviceDriver::GetDIBits(RetainPtr<CFX_DIBitmap> bitmap,
   uint8_t* output_buffer = bitmap->GetWritableBuffer().data();
   DCHECK(output_buffer);
 
-  SkImageInfo input_info =
-      SkImageInfo::Make(m_pBitmap->GetWidth(), m_pBitmap->GetHeight(),
-                        SkColorType::kN32_SkColorType, kPremul_SkAlphaType);
+  SkImageInfo input_info = m_pCanvas->imageInfo();
   sk_sp<SkImage> input = SkImages::RasterFromPixmap(
       SkPixmap(input_info, input_buffer, m_pBitmap->GetPitch()),
       /*rasterReleaseProc=*/nullptr, /*releaseContext=*/nullptr);
 
+  CHECK_EQ(32, bitmap->GetBPP());
   SkImageInfo output_info = SkImageInfo::Make(
       bitmap->GetWidth(), bitmap->GetHeight(),
-      Get32BitSkColorType(m_bRgbByteOrder), kPremul_SkAlphaType);
+      Get32BitSkColorType(m_bRgbByteOrder),
+      bitmap->IsPremultiplied() ? kPremul_SkAlphaType : kUnpremul_SkAlphaType);
   sk_sp<SkSurface> output =
       SkSurfaces::WrapPixels(output_info, output_buffer, bitmap->GetPitch());
 
@@ -1384,7 +1375,7 @@ bool CFX_SkiaDeviceDriver::GetDIBits(RetainPtr<CFX_DIBitmap> bitmap,
   return true;
 }
 
-RetainPtr<CFX_DIBitmap> CFX_SkiaDeviceDriver::GetBackDrop() {
+RetainPtr<const CFX_DIBitmap> CFX_SkiaDeviceDriver::GetBackDrop() const {
   return m_pBackdropBitmap;
 }
 
@@ -1437,26 +1428,22 @@ bool CFX_SkiaDeviceDriver::StretchDIBits(RetainPtr<const CFX_DIBBase> bitmap,
                          sampling_options, blend_type);
 }
 
-bool CFX_SkiaDeviceDriver::StartDIBits(
+RenderDeviceDriverIface::StartResult CFX_SkiaDeviceDriver::StartDIBits(
     RetainPtr<const CFX_DIBBase> bitmap,
     float alpha,
     uint32_t color,
     const CFX_Matrix& matrix,
     const FXDIB_ResampleOptions& options,
-    std::unique_ptr<CFX_ImageRenderer>* handle,
     BlendMode blend_type) {
   FX_RECT rect(0, 0, bitmap->GetWidth(), bitmap->GetHeight());
-  return StartDIBitsSkia(std::move(bitmap), rect, alpha, color, matrix, options,
-                         blend_type);
+  bool success = StartDIBitsSkia(std::move(bitmap), rect, alpha, color, matrix,
+                                 options, blend_type);
+  return {success ? Result::kSuccess : Result::kFailure, nullptr};
 }
 
-bool CFX_SkiaDeviceDriver::ContinueDIBits(CFX_ImageRenderer* handle,
-                                          PauseIndicatorIface* pPause) {
-  return false;
-}
-
-void CFX_DIBitmap::UnPreMultiply() {
-  if (m_nFormat == Format::kUnPreMultiplied || GetBPP() != 32) {
+void CFX_DIBitmap::PreMultiply() {
+  CHECK(CFX_DefaultRenderDevice::UseSkiaRenderer());
+  if (GetFormat() != FXDIB_Format::kBgra) {
     return;
   }
 
@@ -1465,7 +1452,31 @@ void CFX_DIBitmap::UnPreMultiply() {
     return;
   }
 
-  m_nFormat = Format::kUnPreMultiplied;
+  SetFormat(FXDIB_Format::kBgraPremul);
+  int height = GetHeight();
+  int width = GetWidth();
+  int row_bytes = GetPitch();
+  SkImageInfo premultiplied_info =
+      SkImageInfo::Make(width, height, kN32_SkColorType, kPremul_SkAlphaType);
+  SkPixmap premultiplied(premultiplied_info, buffer, row_bytes);
+  SkImageInfo unpremultiplied_info =
+      SkImageInfo::Make(width, height, kN32_SkColorType, kUnpremul_SkAlphaType);
+  SkPixmap unpremultiplied(unpremultiplied_info, buffer, row_bytes);
+  unpremultiplied.readPixels(premultiplied);
+}
+
+void CFX_DIBitmap::UnPreMultiply() {
+  CHECK(CFX_DefaultRenderDevice::UseSkiaRenderer());
+  if (GetFormat() != FXDIB_Format::kBgraPremul) {
+    return;
+  }
+
+  void* buffer = GetWritableBuffer().data();
+  if (!buffer) {
+    return;
+  }
+
+  SetFormat(FXDIB_Format::kBgra);
   int height = GetHeight();
   int width = GetWidth();
   int row_bytes = GetPitch();
@@ -1478,14 +1489,6 @@ void CFX_DIBitmap::UnPreMultiply() {
   premultiplied.readPixels(unpremultiplied);
 }
 
-void CFX_DIBitmap::ForcePreMultiply() {
-  m_nFormat = Format::kPreMultiplied;
-}
-
-bool CFX_DIBitmap::IsPremultiplied() const {
-  return m_nFormat == Format::kPreMultiplied;
-}
-
 bool CFX_SkiaDeviceDriver::DrawBitsWithMask(RetainPtr<const CFX_DIBBase> bitmap,
                                             RetainPtr<const CFX_DIBBase> mask,
                                             float alpha,
@@ -1551,20 +1554,18 @@ void CFX_SkiaDeviceDriver::SetGroupKnockout(bool group_knockout) {
   m_bGroupKnockout = group_knockout;
 }
 
-bool CFX_SkiaDeviceDriver::SyncInternalBitmaps() {
+void CFX_SkiaDeviceDriver::SyncInternalBitmaps() {
   if (!m_pOriginalBitmap) {
-    return true;
+    return;
   }
 
   int width = m_pOriginalBitmap->GetWidth();
   int height = m_pOriginalBitmap->GetHeight();
   DCHECK_EQ(width, m_pBitmap->GetWidth());
   DCHECK_EQ(height, m_pBitmap->GetHeight());
-  DCHECK_EQ(FXDIB_Format::kRgb, m_pOriginalBitmap->GetFormat());
-  m_pOriginalBitmap->TransferBitmap(/*dest_left=*/0, /*dest_top=*/0, width,
-                                    height, m_pBitmap, /*src_left=*/0,
+  DCHECK_EQ(FXDIB_Format::kBgr, m_pOriginalBitmap->GetFormat());
+  m_pOriginalBitmap->TransferBitmap(width, height, m_pBitmap, /*src_left=*/0,
                                     /*src_top=*/0);
-  return true;
 }
 
 void CFX_SkiaDeviceDriver::Clear(uint32_t color) {
@@ -1626,6 +1627,12 @@ bool CFX_SkiaDeviceDriver::StartDIBitsSkia(RetainPtr<const CFX_DIBBase> bitmap,
   return true;
 }
 
+void CFX_SkiaDeviceDriver::DrawPathImpl(const SkPath& path,
+                                        const SkPaint& paint) {
+  DebugShowSkiaDrawPath(this, m_pCanvas, paint, path);
+  m_pCanvas->drawPath(path, paint);
+}
+
 CFX_SkiaDeviceDriver::CharDetail::CharDetail() = default;
 CFX_SkiaDeviceDriver::CharDetail::~CharDetail() = default;
 
@@ -1649,7 +1656,7 @@ bool CFX_DefaultRenderDevice::AttachSkiaImpl(
   return true;
 }
 
-bool CFX_DefaultRenderDevice::AttachCanvas(SkCanvas* canvas) {
+bool CFX_DefaultRenderDevice::AttachCanvas(SkCanvas& canvas) {
   auto driver = CFX_SkiaDeviceDriver::Create(canvas);
   if (!driver) {
     return false;
